/**
 * Created by xiaohe on 2018/5/7.
 */
 const path = require('path');
 const cwd = process.cwd();
 const webpack = require('webpack');
 require("react-hot-loader");
 const project = require(path.join(cwd, `/config/Project.config.js`));
 

 const HappyPack = require('happypack');
 const ParallelUglifyPlugin = require('webpack-parallel-uglify-plugin');
 
 
 module.exports = {
     cache: true,
     entry: {
         app: ['babel-polyfill', 'react-hot-loader/patch', `./${project.publicPath}/entrance/app.js`]
     },
     optimization: {
         splitChunks: {
             chunks: 'all',
             cacheGroups: {
                 commons: {
                     test: /[\\/]node_modules[\\/]/,
                     name: "vendors",
                     chunks: "all"
                 }
             }
         }
     },
     module: {
         rules: [
             {
                 test: /\.(js|jsx)$/i,
                 use: ['happypack/loader?id=babel'],
                 //排除路径
                 exclude: path.join(cwd, 'node_modules'),
                 //读取路径
                 include: path.join(cwd, `${project.publicPath}`)
             },
 
 
             //音视频处理
             {
                 test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/i,
                 loader: 'url-loader',
                 options: {
                     limit: 10000,
                     name: './media/[name].[ext]',
                 }
             }, {
                 test: /\.csv$/i,
                 loader: 'csv-loader',
                 options: {
                     dynamicTyping: true,
                     header: true,
                     skipEmptyLines: true
                 }
             }
         ]
     },
     resolve: {
         /*设置默认读取的后缀*/
         extensions: ['.js', '.jsx', '.scss', '.json', '.css'],
         /*设置优先读取的模块*/
         modules: [path.join(cwd, `./${project.publicPath}/`), 'node_modules'],
         /*设置整体别名*/
         alias: {
             //图片别名
             'img': path.join(cwd, `./${project.publicPath}/public/images`),
             //多媒体别名
             'media': path.join(cwd, `./${project.publicPath}/public/media`),
             //字体别名
             'font': path.join(cwd, `./${project.publicPath}/public/font`),
             //data别名
             'data': path.join(cwd, `./${project.publicPath}/public/data`),
             //mock别名
             'mock': path.join(cwd, `./${project.publicPath}/mock`),
             //action别名
             'action': path.join(cwd, `./${project.publicPath}/redux/Action`),
             //reducer别名
             'reducer': path.join(cwd, `./${project.publicPath}/redux/Reducer`),
             //saga别名
             'saga': path.join(cwd, `./${project.publicPath}/redux/Saga`),
             //store别名
             'store': path.join(cwd, `./${project.publicPath}/redux/Store`),
             //容器别名
             'con': path.join(cwd, `./${project.publicPath}/containers`),
             //组件别名
             'com': path.join(cwd, `./${project.publicPath}/component`),
             //公用方法
             'method': path.join(cwd, `./${project.publicPath}/public/method`),
             //第三方的一些js
             'js': path.join(cwd, `./${project.publicPath}/public/js`),
             //config基本设置
             'config': path.join(cwd, 'config')
         }
     },
     plugins: [
 
         /*设置全局公用声明第三方库*/
         new webpack.ProvidePlugin(project.providePlugin),
 
         /*使用HappyPack实例化*/
         new HappyPack({
             // 用唯一的标识符id来代表当前的HappyPack 处理一类特定的文件
             id: 'babel',
             // 如何处理.js文件，用法和Loader配置是一样的
             loaders: [{
                 loader: "babel-loader",
                 options: {
                     cacheDirectory: true
                 }
             }, 'react-hot-loader/webpack']
         })
         /*,
         new ParallelUglifyPlugin({})*/
     ]
 };
 
 